@@ -2,7 +2,7 @@
 /*
  * This file is part of gps-data-logger-widget
  *
- * Copyright (C) 2009 Marcell Lengyel <marcell@maemo.org>
+ * Copyright (C) 2009-2011 Marcell Lengyel <marcell@maemo.org>
  *
  */
 
@@ -24,28 +24,26 @@
 
 #include <hildon/hildon.h>
 #include <libhildondesktop/libhildondesktop.h>
-#include <hildon-fm-2/hildon/hildon-file-chooser-dialog.h>
 #include <hildon/hildon-banner.h>
 
-#include "gps-data-logger-widget.h"
+/* screen on/off */
+#include <dbus/dbus.h>
+#include <dbus/dbus-glib-lowlevel.h> /* dbus_connection_setup_with_g_main */
+#include <mce/dbus-names.h>
+#include <mce/mode-names.h>
 
-#define PADDING 10
-#define THEME_DIR "/etc/hildon/theme/images/"
-#define BUTTON_BG THEME_DIR "ApplicationShortcutApplet.png"
-#define BUTTON_PRESSED_BG THEME_DIR "ApplicationShortcutAppletPressed.png"
-#define REC_ICON "camera_video_recording"
-#define STOP_ICON "camera_video_stop"
-#define PAUSE_ICON "camera_video_pause"
-#define GPS_REFRESH_ICON "general_refresh"
-#define GPS_ON_ICON "gps_location"
-#define GPS_OFF_ICON "gps_not_connected"
+#include <location/location-distance-utils.h> /* location_distance_between */
+#include "gps-data-logger-widget.h"
+#include "gps-data-logger-gfx.h"
+#include "settings-dialog.h"
 
 #define OSSO_GDL_NAME    "gps_data_logger"
 #define OSSO_GDL_SERVICE "org.maemo."OSSO_GDL_NAME
 #define OSSO_GDL_OBJECT  "/org/maemo/"OSSO_GDL_NAME
 #define OSSO_GDL_IFACE   "org.maemo."OSSO_GDL_NAME
 
-#define AUTO_PAUSE_TIME_MULTIPLIER 5
+#define MCE_MATCH_RULE "type='signal',interface='" MCE_SIGNAL_IF \
+	                       "',member='" MCE_DISPLAY_SIG "'"
 
 HD_DEFINE_PLUGIN_MODULE (GpsDataLoggerWidget, gps_data_logger_widget, HD_TYPE_HOME_PLUGIN_ITEM)
 
@@ -54,383 +52,6 @@ static gboolean manual_on_stop (gpointer data);
 static void on_stop(LocationGPSDControl *control, gpointer data);
 	
 /* -------------------------------------------------------------------- */
-/* Settings
- */
-
-static void
-apply_settings (GpsDataLoggerWidget * self)
-{
-    LocationGPSDControlMethod my_method = 0;
-    gint my_interval;
-
-    if (self->settings->method_agnss)
-	    my_method = my_method | LOCATION_METHOD_AGNSS;
-    if (self->settings->method_gnss)
-        my_method = my_method | LOCATION_METHOD_GNSS;
-    if (self->settings->method_acwp)
-        my_method = my_method | LOCATION_METHOD_ACWP;
-    if (self->settings->method_cwp)
-        my_method = my_method | LOCATION_METHOD_CWP;
-    if (self->settings->interval == 0)
-        my_interval = LOCATION_INTERVAL_1S;
-    else if (self->settings->interval == 1)
-        my_interval = LOCATION_INTERVAL_2S;
-    else if (self->settings->interval == 2)
-        my_interval = LOCATION_INTERVAL_5S;
-    else if (self->settings->interval == 3)
-        my_interval = LOCATION_INTERVAL_10S;
-    else if (self->settings->interval == 4)
-        my_interval = LOCATION_INTERVAL_20S;
-    else if (self->settings->interval == 5)
-        my_interval = LOCATION_INTERVAL_30S;
-    else if (self->settings->interval == 6)
-        my_interval = LOCATION_INTERVAL_60S;
-    else if (self->settings->interval == 7)
-        my_interval = LOCATION_INTERVAL_120S;
-
-    g_object_set(G_OBJECT (self->control),
-                 "preferred-method", my_method,
-                 "preferred-interval", my_interval,
-                 NULL);
-}
-
-void
-on_btn_path (GtkWidget * widget, gpointer user_data)
-{
-    GtkWidget *chooser;
-    GtkDialog *dialog = GTK_DIALOG(user_data);
-
-    chooser = hildon_file_chooser_dialog_new(GTK_WINDOW(dialog),
-		    GTK_FILE_CHOOSER_ACTION_SELECT_FOLDER);
-    /* Not in GTK 2.14 :(
-    gtk_file_chooser_set_create_folders(GTK_FILE_CHOOSER (chooser),
-		    TRUE); */
-    gtk_file_chooser_set_current_folder(GTK_FILE_CHOOSER (chooser),
-		    "/home/user/MyDocs");
-    if (gtk_dialog_run (GTK_DIALOG (chooser)) == GTK_RESPONSE_OK) {
-        hildon_button_set_value(HILDON_BUTTON(widget),
-		 gtk_file_chooser_get_filename (GTK_FILE_CHOOSER (chooser)));
-	gtk_widget_destroy (chooser);
-    } else {
-	gtk_widget_destroy (chooser);
-    }
-}
-
-static void
-on_settings_dialog_response (GtkDialog *dialog,
-  gint response_id, gpointer data)
-{
-    GpsDataLoggerWidget *self = GPS_DATA_LOGGER_WIDGET (data);
-    gboolean	new_value, save_needed = FALSE;
-    gint 	new_int;
-    
-    g_assert (self->settings_dialog);
-    g_assert (self->settings_dialog == GTK_WIDGET (dialog));
-
-    if (response_id == GTK_RESPONSE_OK) {
-        /* save settings */
-	new_value = hildon_check_button_get_active(
-	            HILDON_CHECK_BUTTON(self->chk_method_agnss));
-        if (self->settings->method_agnss != new_value) {
-	    save_needed = TRUE;
-	    self->settings->method_agnss = new_value;
-	}
-	new_value = hildon_check_button_get_active(
-		    HILDON_CHECK_BUTTON(self->chk_method_gnss));
-        if (self->settings->method_gnss != new_value) {
-	    save_needed = TRUE;
-	    self->settings->method_gnss = new_value;
-	}
-        new_value = hildon_check_button_get_active(
-		    HILDON_CHECK_BUTTON(self->chk_method_acwp));
-        if (self->settings->method_acwp != new_value) {
-	    save_needed = TRUE;
-	    self->settings->method_acwp = new_value;
-	}
-        new_value = hildon_check_button_get_active(
-		    HILDON_CHECK_BUTTON(self->chk_method_cwp));
-        if (self->settings->method_cwp != new_value) {
-	    save_needed = TRUE;
-	    self->settings->method_cwp = new_value;
-	}
-        new_int = hildon_picker_button_get_active(
-		  HILDON_PICKER_BUTTON(self->pkr_interval));
-	if (self->settings->interval != new_int) {
-            save_needed = TRUE;
-	    self->settings->interval = new_int;
-	}
-	new_value = hildon_check_button_get_active(
-		    HILDON_CHECK_BUTTON(self->chb_auto_pause));
-        if (self->settings->auto_pause != new_value) {
-	    save_needed = TRUE;
-	    self->settings->auto_pause = new_value;
-	}
-        new_int = hildon_picker_button_get_active(
-		  HILDON_PICKER_BUTTON(self->pkr_pause_speed_threshold));
-	if (self->settings->pause_speed_threshold != new_int) {
-            save_needed = TRUE;
-	    self->settings->pause_speed_threshold = new_int;
-	}
-        new_int = hildon_picker_button_get_active(
-		  HILDON_PICKER_BUTTON(self->pkr_pause_time_threshold));
-	if (self->settings->pause_time_threshold != new_int) {
-            save_needed = TRUE;
-	    self->settings->pause_time_threshold = new_int;
-	}
-	
-	if (g_ascii_strcasecmp(self->settings->log_folder,
-                               hildon_button_get_value(
-			       HILDON_BUTTON(self->btn_path))) != 0) {
-	    g_stpcpy(self->settings->log_folder,
-		     hildon_button_get_value(
-		     HILDON_BUTTON(self->btn_path)));
-	    save_needed = TRUE;
-	}
-	if (save_needed) {
-	    settings_save(self->settings);
-	    /* apply settings */
-	    apply_settings(self);
-	}
-    } 
-    gtk_widget_destroy (self->settings_dialog);
-    self->settings_dialog = NULL;
-}
-	      
-	
-GtkDialog *
-settings_dialog_new (GpsDataLoggerWidget * self)
-{
-    GtkWidget *dialog;
-    GtkWidget *vbox;
-    GtkWidget *selector, *speed_selector, *time_selector;
-    GtkWidget *panarea;
-    gchar *text;
-    int i;
-
-    /* dialog title */
-    dialog = gtk_dialog_new_with_buttons (_("GPS Data Logger Widget Settings"),
-					  NULL, /* parent */
-					  GTK_DIALOG_DESTROY_WITH_PARENT |
-					  GTK_DIALOG_NO_SEPARATOR,
-					  GTK_STOCK_SAVE, 
-					  GTK_RESPONSE_OK, NULL);
-    gtk_widget_set_size_request (dialog, 800, 350);
-    vbox = gtk_vbox_new (FALSE, 0);
-    gtk_box_pack_start (GTK_BOX (vbox),
-                        gtk_label_new (_("Location Methods")),
-                        TRUE, TRUE, 4);
-    /* AGNSS */
-    self->chk_method_agnss = hildon_check_button_new (HILDON_SIZE_FINGER_HEIGHT);
-    hildon_check_button_set_active(HILDON_CHECK_BUTTON(self->chk_method_agnss),
-		    		self->settings->method_agnss);
-    gtk_button_set_label (GTK_BUTTON (self->chk_method_agnss),
-                          _("Assisted GPS"));
-    gtk_button_set_alignment (GTK_BUTTON (self->chk_method_agnss),
-                              0.0, 0.5);
-    gtk_box_pack_start (GTK_BOX (vbox),
-                        self->chk_method_agnss, TRUE, TRUE, 4);
-    /* GNSS */
-    self->chk_method_gnss = hildon_check_button_new (HILDON_SIZE_FINGER_HEIGHT);
-    hildon_check_button_set_active(HILDON_CHECK_BUTTON(self->chk_method_gnss),
-		    		self->settings->method_gnss);
-    gtk_button_set_label (GTK_BUTTON (self->chk_method_gnss),
-                          _("GPS"));
-    gtk_button_set_alignment (GTK_BUTTON (self->chk_method_gnss),
-                              0.0, 0.5);
-    gtk_box_pack_start (GTK_BOX (vbox),
-                        self->chk_method_gnss, TRUE, TRUE, 4);
-    /* ACWP */
-    self->chk_method_acwp = hildon_check_button_new (HILDON_SIZE_FINGER_HEIGHT);
-    hildon_check_button_set_active(HILDON_CHECK_BUTTON(self->chk_method_acwp),
-		    		self->settings->method_acwp);
-    gtk_button_set_label (GTK_BUTTON (self->chk_method_acwp),
-                          _("Assisted, using cellular base stations"));
-    gtk_button_set_alignment (GTK_BUTTON (self->chk_method_acwp),
-                              0.0, 0.5);
-    gtk_box_pack_start (GTK_BOX (vbox),
-                        self->chk_method_acwp, TRUE, TRUE, 4);
-
-    /* CWP */
-    self->chk_method_cwp = hildon_check_button_new (HILDON_SIZE_FINGER_HEIGHT);
-    hildon_check_button_set_active(HILDON_CHECK_BUTTON(self->chk_method_cwp),
-		    		self->settings->method_cwp);
-    gtk_button_set_label (GTK_BUTTON (self->chk_method_cwp),
-                          _("Center of the current country"));
-    gtk_button_set_alignment (GTK_BUTTON (self->chk_method_cwp),
-                              0.0, 0.5);
-    gtk_box_pack_start (GTK_BOX (vbox),
-                        self->chk_method_cwp, TRUE, TRUE, 4);
-    /* Interval */
-    gtk_box_pack_start (GTK_BOX (vbox),
-                        gtk_label_new (_("Interval")),
-                        TRUE, TRUE, 4);
-    /* Interval picker */
-    self->pkr_interval = hildon_picker_button_new
-            (HILDON_SIZE_FINGER_HEIGHT,
-             HILDON_BUTTON_ARRANGEMENT_VERTICAL);
-    gtk_button_set_alignment (GTK_BUTTON (self->pkr_interval), 0.0, 0.5);
-    hildon_button_set_title (HILDON_BUTTON (self->pkr_interval),
-                             _("Logging interval"));
-    selector = hildon_touch_selector_new_text ();
-    hildon_touch_selector_append_text(HILDON_TOUCH_SELECTOR (selector),
-                                      _(" 1 second"));
-    hildon_touch_selector_append_text(HILDON_TOUCH_SELECTOR (selector),
-                                      _(" 2 seconds"));
-    hildon_touch_selector_append_text(HILDON_TOUCH_SELECTOR (selector),
-                                      _(" 5 seconds"));
-    hildon_touch_selector_append_text(HILDON_TOUCH_SELECTOR (selector),
-                                      _("10 seconds"));
-    hildon_touch_selector_append_text(HILDON_TOUCH_SELECTOR (selector),
-                                      _("20 seconds"));
-    hildon_touch_selector_append_text(HILDON_TOUCH_SELECTOR (selector),
-                                      _("30 seconds"));
-    hildon_touch_selector_append_text(HILDON_TOUCH_SELECTOR (selector),
-                                      _("60 seconds"));
-    hildon_touch_selector_append_text(HILDON_TOUCH_SELECTOR (selector),
-                                      _("120 seconds"));
-    hildon_picker_button_set_selector(HILDON_PICKER_BUTTON
-             (self->pkr_interval), HILDON_TOUCH_SELECTOR (selector));
-    gtk_box_pack_start (GTK_BOX (vbox),
-                        self->pkr_interval, TRUE, TRUE, 5);
-    hildon_touch_selector_set_active (HILDON_TOUCH_SELECTOR (selector),
-                                      0, self->settings->interval);
-
-    /* Auto pause */
-    self->chb_auto_pause = hildon_check_button_new (HILDON_SIZE_FINGER_HEIGHT);
-    gtk_button_set_label (GTK_BUTTON (self->chb_auto_pause), _("Auto pause enabled"));
-    hildon_check_button_set_active(HILDON_CHECK_BUTTON(self->chb_auto_pause),
-		    		self->settings->auto_pause);
-    gtk_box_pack_start (GTK_BOX (vbox),
-		        self->chb_auto_pause, TRUE, TRUE, 5);
-    /* speed threshold */
-    self->pkr_pause_speed_threshold = hildon_picker_button_new
-	               (HILDON_SIZE_FINGER_HEIGHT,
-		        HILDON_BUTTON_ARRANGEMENT_VERTICAL);
-    gtk_button_set_alignment (GTK_BUTTON (self->pkr_pause_speed_threshold), 
-		    		0.0, 0.5);
-    hildon_button_set_title (HILDON_BUTTON (self->pkr_pause_speed_threshold),
-		               _("Pause if speed is less than"));
-    speed_selector = hildon_touch_selector_new_text ();
-    text = g_malloc0(10);
-    for (i = 1; i <= 30; i++ ) {
-        g_sprintf(text, "%d km/h", i);
-	hildon_touch_selector_append_text(HILDON_TOUCH_SELECTOR (speed_selector),
-			text);
-    }
-    g_free(text);
-    hildon_picker_button_set_selector(HILDON_PICKER_BUTTON
-             (self->pkr_pause_speed_threshold), 
-	     HILDON_TOUCH_SELECTOR (speed_selector));
-    gtk_box_pack_start (GTK_BOX (vbox),
-                        self->pkr_pause_speed_threshold, TRUE, TRUE, 5);
-    hildon_touch_selector_set_active (HILDON_TOUCH_SELECTOR (speed_selector),
-                                      0, self->settings->pause_speed_threshold);
-    /* interval threshold */
-    self->pkr_pause_time_threshold = hildon_picker_button_new
-	               (HILDON_SIZE_FINGER_HEIGHT,
-		        HILDON_BUTTON_ARRANGEMENT_VERTICAL);
-    gtk_button_set_alignment (GTK_BUTTON (self->pkr_pause_time_threshold), 
-		    		0.0, 0.5);
-    hildon_button_set_title (HILDON_BUTTON (self->pkr_pause_time_threshold),
-		               _("For at least"));
-    time_selector = hildon_touch_selector_new_text ();
-    text = g_malloc0(25);
-    for (i = 1; i <= 25; i++ ) {
-        g_sprintf(text, _("%d seconds"), i * AUTO_PAUSE_TIME_MULTIPLIER);
-	hildon_touch_selector_append_text(HILDON_TOUCH_SELECTOR (time_selector),
-			text);
-    }
-    g_free(text);
-    hildon_picker_button_set_selector(HILDON_PICKER_BUTTON
-             (self->pkr_pause_time_threshold), 
-	     HILDON_TOUCH_SELECTOR (time_selector));
-    gtk_box_pack_start (GTK_BOX (vbox),
-                        self->pkr_pause_time_threshold, TRUE, TRUE, 5);
-    hildon_touch_selector_set_active (HILDON_TOUCH_SELECTOR (time_selector),
-                                      0, self->settings->pause_time_threshold);
-    
-    
-    /* Log */
-    gtk_box_pack_start (GTK_BOX (vbox),
-                        gtk_label_new (_("Log")),
-                        TRUE, TRUE, 4);
-    /* Log path */
-    self->btn_path = hildon_button_new (HILDON_SIZE_FINGER_HEIGHT,
-		                  HILDON_BUTTON_ARRANGEMENT_VERTICAL);
-    gtk_button_set_alignment (GTK_BUTTON (self->btn_path), 0.0, 0.5);
-    hildon_button_set_title(HILDON_BUTTON(self->btn_path),
-		            _("Folder to store the logs"));
-    hildon_button_set_value(HILDON_BUTTON(self->btn_path),
-		            self->settings->log_folder);
-    gtk_box_pack_start (GTK_BOX (vbox),
-		        self->btn_path, TRUE, TRUE, 5);
-    g_signal_connect(self->btn_path, "clicked",
-		     G_CALLBACK (on_btn_path), dialog);    
-    /* */
-    panarea = hildon_pannable_area_new ();
-    g_object_set (panarea, "initial-hint", TRUE, NULL);
-    hildon_pannable_area_add_with_viewport
-            (HILDON_PANNABLE_AREA (panarea), GTK_WIDGET (vbox));
-    gtk_box_pack_start (GTK_BOX
-                        (GTK_DIALOG (dialog)->vbox), panarea,
-                        TRUE, TRUE, 5);
-    gtk_widget_show_all (dialog);
-    hildon_pannable_area_scroll_to (HILDON_PANNABLE_AREA (panarea), -1,
-                                    0);
-    return GTK_DIALOG (dialog);
-}
-
-void
-on_settings_dialog_show (GtkWidget *widget,
-			 gpointer data)
-{
-    GpsDataLoggerWidget *self = GPS_DATA_LOGGER_WIDGET (data);
-    if (!self->settings_dialog) {
-    	self->settings_dialog = GTK_WIDGET (settings_dialog_new (self));
-        g_signal_connect (self->settings_dialog, "response",
-                          G_CALLBACK (on_settings_dialog_response),
-                          self);
-    } else {
-	gtk_widget_show (self->settings_dialog);
-    }
-}
-
-/* -------------------------------------------------------------------- */
-/* Pixbufs and icons
- */
-
-static void
-free_pixbuf(GdkPixbuf **pixbuf)
-{
-    if (*pixbuf)
-    {
-        g_object_unref(*pixbuf);
-        *pixbuf = NULL;
-    }
-}
-
-static void
-free_theme_pixbufs(GpsDataLoggerWidget * self)
-{
-    free_pixbuf(&self->button_bg);
-    free_pixbuf(&self->button_pressed_bg);
-}
-
-static void
-load_theme_pixbufs(GpsDataLoggerWidget * self)
-{
-    self->button_bg = gdk_pixbuf_new_from_file(BUTTON_BG, NULL);
-    self->button_pressed_bg = gdk_pixbuf_new_from_file(BUTTON_PRESSED_BG, NULL);
-}
-
-static gboolean
-theme_pixbufs_loaded(GpsDataLoggerWidget * self)
-{
-    if (self->button_bg && self->button_pressed_bg)
-        return TRUE;
-
-    return FALSE;
-}
 
 static gboolean
 on_style_set(GtkWidget *widget, GtkStyle *previous_style, gpointer data)
@@ -484,18 +105,7 @@ load_icons(GpsDataLoggerWidget * self)
 
 /* -------------------------------------------------------------------- */
 
-static gboolean
-on_widget_leave(GtkWidget* widget, GdkEventCrossing *event, gpointer data)
-{
-    GpsDataLoggerWidget *self = GPS_DATA_LOGGER_WIDGET (data);
-
-    self->rec_button_down = FALSE;
-    self->stop_button_down = FALSE;
-    gtk_widget_queue_draw(widget);
-
-    return FALSE;
-}
-
+/* DEBUG log */
 
 #ifdef DEBUG
 static void
@@ -506,8 +116,9 @@ debuglog_close(GpsDataLoggerWidget * self)
 	self->logfile = NULL;
     }
 }
+#endif
 
-
+#ifdef DEBUG
 static void
 debuglog_start(GpsDataLoggerWidget * self)
 {
@@ -536,6 +147,10 @@ debuglog_start(GpsDataLoggerWidget * self)
 }
 #endif
 
+/* -------------------------------------------------------------------- */
+
+/* GPX file */
+
 static void
 logfile_close(GpsDataLoggerWidget * self)
 {
@@ -620,6 +235,20 @@ logfile_start(GpsDataLoggerWidget * self)
     /* g_free(path); */
 }
 
+/* -------------------------------------------------------------------- */
+
+static gboolean
+on_widget_leave(GtkWidget* widget, GdkEventCrossing *event, gpointer data)
+{
+    GpsDataLoggerWidget *self = GPS_DATA_LOGGER_WIDGET (data);
+
+    self->rec_button_down = FALSE;
+    self->stop_button_down = FALSE;
+    gtk_widget_queue_draw(widget);
+
+    return FALSE;
+}
+
 static void
 update_status_and_buttons (GpsDataLoggerWidget * self)
 {
@@ -637,6 +266,7 @@ update_status_and_buttons (GpsDataLoggerWidget * self)
 		gtk_image_clear(GTK_IMAGE(self->auto_pause_status));
 		self->rec_button_enabled = FALSE;
 		self->stop_button_enabled = TRUE;
+		self->distance = 0.0;
 #ifndef __arm__
 		g_print(" starting: loading new icon, disabling rec\n");
 #endif
@@ -919,6 +549,32 @@ on_button_press(GtkWidget* widget, GdkEventButton *button_event, gpointer data)
     return retval;
 }
 
+void gps_data_logger_update_distance(gpointer data)
+{
+	GpsDataLoggerWidget *self = GPS_DATA_LOGGER_WIDGET (data);
+	gchar *dist = NULL;
+
+#ifdef DEBUG
+        if (self->logfile) {
+            g_fprintf(self->logfile, "%s -- enter, distance: %.2f km\n", __func__, self->distance);
+	    fflush(self->logfile);
+        }
+#endif
+	/* do this only if the widget is visible! */
+	dist = g_malloc(25);
+	if (self->distance < 1) {
+	    g_snprintf(dist, 25, "%.0f m", self->distance * 1000);
+	} else if (self->distance < 10) {
+	    g_snprintf(dist, 25, "%.2f km", self->distance);
+	} else {
+	    g_snprintf(dist, 25, "%.1f km", self->distance);
+	}
+	gtk_label_set_text(GTK_LABEL(self->distance_value_string), dist);
+}
+
+/* -------------------------------------------------------------------- */
+/* liblocation */
+
 static void
 on_error(LocationGPSDControl *control, gpointer data)
 {
@@ -1007,6 +663,7 @@ on_stop(LocationGPSDControl *control, gpointer data)
 	}
 }
 
+
 static void
 on_changed(LocationGPSDevice *device, gpointer data)
 {
@@ -1067,6 +724,8 @@ on_changed(LocationGPSDevice *device, gpointer data)
                                 _("on_changed, going to RECORDING state"));
                   
                             self->app_state = APP_STATE_RECORDING;
+			    self->prev_lat = 0.0;
+			    self->prev_lon = 0.0;
 #ifndef __arm__
                             g_print("on_change : calling update_status_and_buttons\n");
 #endif
@@ -1106,6 +765,43 @@ on_changed(LocationGPSDevice *device, gpointer data)
 			    }
 			}
 			g_fprintf(self->file, string_buf);
+			/* calculate distance */
+			if (self->prev_lat != 0.0 && self->prev_lon != 0.0
+					&& self->app_state == APP_STATE_RECORDING) {
+			    /*
+			    self->distance += location_distance_between(self->prev_lat,
+									self->prev_lon,
+									device->fix->latitude,
+									device->fix->longitude);
+									*/
+			    double delta;
+			    delta = location_distance_between(self->prev_lat,
+									self->prev_lon,
+									device->fix->latitude,
+									device->fix->longitude);
+			    /* if the delta is smaller then the possible GPS error, drop
+			     * the data, and use the previous prev_lat and prev_lon */
+			    if ((delta * 1000) > (device->fix->eph / 100.0)) {
+			        self->distance += delta;
+				self->prev_lat = device->fix->latitude;
+				self->prev_lon = device->fix->longitude;
+
+			        /* if the screen is on, update the distance counter */
+			        if (self->display_status)
+			            gps_data_logger_update_distance(self);	
+			    }
+#ifdef DEBUG
+                            if (self->logfile) {
+                                g_fprintf(self->logfile, "  distance: %f, delta: %f\n", 
+						self->distance, delta);
+	                        fflush(self->logfile);
+                            }
+#endif
+
+			} else {
+			    self->prev_lat = device->fix->latitude;
+			    self->prev_lon = device->fix->longitude;
+			}
 			if (device->fix->fields & LOCATION_GPS_DEVICE_ALTITUDE_SET) {
 			    g_snprintf(string_buf, 256, "  <ele>%.1f</ele>\n",
 						device->fix->altitude);
@@ -1216,71 +912,7 @@ on_changed(LocationGPSDevice *device, gpointer data)
 #endif
 }
 
-static void
-draw_pixbuf(cairo_t *cr, GdkPixbuf *pbuf, int x, int y, gboolean dimmed)
-{
-        int pbuf_width, pbuf_height;
-
-        pbuf_width = gdk_pixbuf_get_width(pbuf);
-        pbuf_height = gdk_pixbuf_get_height(pbuf);
-
-        gdk_cairo_set_source_pixbuf(cr, pbuf, 
-			            x - (pbuf_width / 2),
-                                    y - (pbuf_height / 2));
-	if (dimmed)
-	    cairo_paint_with_alpha(cr, 0.3);
-	else
-            cairo_paint(cr);
-}
-
-static void
-draw_rounded_rectangle(cairo_t *cr, double x, double y,
-                double width, double height, double rad)
-{
-        cairo_move_to(cr, x+rad, y);
-        cairo_line_to(cr, x+width-rad, y);
-        cairo_curve_to(cr, x+width, y, x+width, y, x+width, y+rad);
-        cairo_line_to(cr, x+width, y+height-rad);
-        cairo_curve_to(cr, x+width, y+height, x+width, y+height, x+width-rad, y+height);
-        cairo_line_to(cr, x+rad, y+height);
-        cairo_curve_to(cr, x, y+height, x, y+height, x, y+height-rad);
-        cairo_line_to(cr, x, y+rad);
-        cairo_curve_to(cr, x, y, x, y, x+rad, y);
-        cairo_close_path(cr);
-}
-
-static void
-draw_background(cairo_t *cr, GdkRectangle *area, gint border, gdouble transparency)
-{
-        cairo_pattern_t *grad;
-
-	gdk_cairo_rectangle(cr, area);
-        cairo_clip(cr);
-
-        cairo_set_operator(cr, CAIRO_OPERATOR_OVER);
-
-        /* Gradient background */
-        grad = cairo_pattern_create_linear(area->x+border,
-                        area->y+border, area->x+border,
-                        area->y+area->height-(2*border));
-        cairo_pattern_add_color_stop_rgba(grad, 0, 0.4, 0.4, 0.4, transparency);
-        cairo_pattern_add_color_stop_rgba(grad, 1, 0.05, 0.05, 0.05, transparency);
-
-        /* Draw border */
-        cairo_set_line_width(cr, 2);
-        draw_rounded_rectangle(cr, area->x+border,
-                        area->y+border, area->width-(2*border),
-                        area->height-(2*border), 10.0);
-        cairo_set_source_rgba(cr, 0.4, 0.4, 0.4, transparency);
-        cairo_stroke_preserve(cr);
-
-        /* Fill background with gradient */
-        cairo_set_source(cr, grad);
-        cairo_fill(cr);
-
-        cairo_pattern_destroy(grad);
-}
-
+/* -------------------------------------------------------------------- */
 
 static gboolean
 on_background_expose(GtkWidget *widget, GdkEventExpose *expose_event, gpointer data)
@@ -1364,7 +996,9 @@ on_button_expose(GtkWidget *widget, GdkEventExpose *expose_event, gpointer data)
     return FALSE;
 }
 
+/* -------------------------------------------------------------------- */
 /* Callback for hardware D-BUS events */
+
 void hw_event_handler(osso_hw_state_t *state, gpointer data)
 {
     GpsDataLoggerWidget *self = GPS_DATA_LOGGER_WIDGET (data);
@@ -1428,17 +1062,45 @@ void hw_event_handler(osso_hw_state_t *state, gpointer data)
     }
 }
 
+static DBusHandlerResult
+dbus_signal_filter(DBusConnection *conn, DBusMessage *message, gpointer data)
+{
+    GpsDataLoggerWidget *self = GPS_DATA_LOGGER_WIDGET (data);
+    char *display_mode;
+
+    /* if idle, we do not care if screen is on or off */
+    if (self->app_state == APP_STATE_IDLE)
+        return DBUS_HANDLER_RESULT_HANDLED;
+    if (dbus_message_is_signal(message, MCE_SIGNAL_IF, MCE_DISPLAY_SIG)) {
+        if (dbus_message_get_args(message, NULL, DBUS_TYPE_STRING, &display_mode,
+				  DBUS_TYPE_INVALID)) {
+            if (!g_strcmp0(display_mode, MCE_DISPLAY_OFF_STRING)) {
+                self->display_status = FALSE;
+	    } else {
+                self->display_status = TRUE;
+		gps_data_logger_update_distance(self);
+	    }
+#ifndef __arm__
+            g_print("Display status: %d\n", self->display_status);
+#endif
+	}
+	return DBUS_HANDLER_RESULT_HANDLED;
+    } else {
+        return DBUS_HANDLER_RESULT_NOT_YET_HANDLED;
+    }
+}
 
 static void
 gps_data_logger_widget_init (GpsDataLoggerWidget * self)
 {
-    GtkWidget *vbox, *hbox_status, *hbox_buttons;
+    GtkWidget *vbox, *hbox_status, *hbox_buttons, *hbox_distance;
     GtkWidget *status_text;
     GdkScreen *screen;
     GdkColormap *colormap;
     osso_return_t result;
     GString *version = NULL ;
     GString *tmp = NULL;
+    DBusError error;
 
     version = g_string_new(getenv("OSSO_PRODUCT_RELEASE_VERSION"));
 #ifndef __arm__
@@ -1459,7 +1121,47 @@ gps_data_logger_widget_init (GpsDataLoggerWidget * self)
 #ifdef DEBUG
     self->logfile = NULL;
 #endif
+    self->display_status = TRUE;
     self->app_state = APP_STATE_IDLE;
+
+    /* setup dbus for display state information */
+    dbus_error_init(&error);
+    self->dbus_conn = hd_home_plugin_item_get_dbus_connection(
+		      HD_HOME_PLUGIN_ITEM(self), DBUS_BUS_SYSTEM, &error);
+    if (dbus_error_is_set (&error)) {
+#ifndef __arm__
+        g_print("init : hd_home_plugin_item_get_dbus_connection failed.\n");
+#endif
+        hildon_banner_show_information(NULL,
+			NULL,
+			_("Can not get DBus connection!"));
+	dbus_error_free (&error);
+    } else {
+        dbus_connection_setup_with_g_main(self->dbus_conn, NULL);
+	dbus_bus_add_match (self->dbus_conn, MCE_MATCH_RULE, &error);
+	if (dbus_error_is_set (&error)) {
+#ifndef __arm__
+            g_print("init : dbus_bus_add_match for DISPLAY_MODE_SIGNAL failed.\n");
+#endif
+            hildon_banner_show_information(NULL,
+	    		    NULL,
+			    _("Could not add DBus match!"));
+	    dbus_error_free (&error);
+        } else {
+	    if (!dbus_connection_add_filter(self->dbus_conn, dbus_signal_filter,
+				            self, NULL)) {
+#ifndef __arm__
+                g_print("init : dbus_connection_add_filter failed.\n");
+#endif
+                hildon_banner_show_information(NULL,
+  	    		        NULL,
+			        _("Could not add DBus filter!"));
+	    }
+	}
+    }
+    
+    self->file = NULL;			    
+    
     /* Preload all icon pixmaps */
     load_icons(self);
     load_theme_pixbufs(self);
@@ -1507,12 +1209,21 @@ gps_data_logger_widget_init (GpsDataLoggerWidget * self)
 
     hbox_status = gtk_hbox_new (FALSE, 3);
     hbox_buttons = gtk_hbox_new (FALSE, 2);
+
+    self->distance_label = gtk_label_new(_("Distance: "));
+    self->distance_value_string = gtk_label_new(_("0 m"));
+    self->distance = 0;
+    hbox_distance = gtk_hbox_new (FALSE, 2);
+
     gtk_box_pack_start (GTK_BOX (hbox_status), status_text, FALSE, TRUE, 0);
     gtk_box_pack_start (GTK_BOX (hbox_status), self->gps_status, FALSE, FALSE, 0);
     gtk_box_pack_start (GTK_BOX (hbox_status), self->auto_pause_status, FALSE, FALSE, 0);
+    gtk_box_pack_start (GTK_BOX (hbox_distance), self->distance_label, FALSE, TRUE, 0);
+    gtk_box_pack_start (GTK_BOX (hbox_distance), self->distance_value_string, FALSE, FALSE, 0);
     gtk_box_pack_start (GTK_BOX (hbox_buttons), self->button_rec, FALSE, FALSE, 0);
     gtk_box_pack_start (GTK_BOX (hbox_buttons), self->button_stop, FALSE, FALSE, 0);
     gtk_box_pack_start (GTK_BOX (vbox), hbox_status, FALSE, FALSE, 0);
+    gtk_box_pack_start (GTK_BOX (vbox), hbox_distance, FALSE, FALSE, 0);
     gtk_box_pack_start (GTK_BOX (vbox), hbox_buttons, FALSE, FALSE, 0);
      
 #ifndef __arm__
@@ -1628,6 +1339,12 @@ gps_data_logger_widget_finalize (GObject *object)
         osso_deinitialize(self->osso_context);
         self->osso_context = NULL;
     }
+
+    dbus_bus_remove_match (self->dbus_conn, MCE_MATCH_RULE, NULL);
+    dbus_connection_remove_filter (self->dbus_conn, dbus_signal_filter, self);
+    dbus_connection_close (self->dbus_conn);
+    dbus_connection_unref (self->dbus_conn);
+
 		    
     /* Call the base class's implementation: */
     G_OBJECT_CLASS(gps_data_logger_widget_parent_class)->finalize (object);
