@@ -1,7 +1,7 @@
 /*
  * This file is part of gps-data-logger-widget
  *
- * Copyright (C) 2009 Marcell Lengyel <marcell@maemo.org>
+ * Copyright (C) 2009-2011 Marcell Lengyel <marcell@maemo.org>
  *
  * This maemo code example is licensed under a MIT-style license,
  * that can be found in the file called "COPYING" in the package
@@ -22,6 +22,17 @@
 
 G_BEGIN_DECLS
 
+#define PADDING 10
+#define THEME_DIR "/etc/hildon/theme/images/"
+#define BUTTON_BG THEME_DIR "ApplicationShortcutApplet.png"
+#define BUTTON_PRESSED_BG THEME_DIR "ApplicationShortcutAppletPressed.png"
+#define REC_ICON "camera_video_recording"
+#define STOP_ICON "camera_video_stop"
+#define PAUSE_ICON "camera_video_pause"
+#define GPS_REFRESH_ICON "general_refresh"
+#define GPS_ON_ICON "gps_location"
+#define GPS_OFF_ICON "gps_not_connected"
+
 /* Common struct types declarations */
 typedef struct _GpsDataLoggerWidget GpsDataLoggerWidget;
 typedef struct _GpsDataLoggerWidgetClass GpsDataLoggerWidgetClass;
@@ -61,6 +72,7 @@ struct _GpsDataLoggerWidget
 	double		auto_pause_time_mark;
 	LocationGPSDControl *control;
 	LocationGPSDevice *device;
+	DBusConnection	*dbus_conn;
 	FILE		*file;
 	gchar		*path;
 #ifdef DEBUG
@@ -92,12 +104,18 @@ struct _GpsDataLoggerWidget
 	GtkWidget	*auto_pause_status;
 	GtkWidget	*button_rec;
 	GtkWidget	*button_stop;
+	GtkWidget	*distance_label;
+	GtkWidget	*distance_value_string;
+	double		distance;
+	double		prev_lat;
+	double		prev_lon;
 	gboolean	rec_button_down;
 	gboolean	rec_button_enabled;
 	gboolean	pause_button_visible;
 	gboolean	stop_button_down;
 	gboolean	stop_button_enabled;
 	gboolean	has_fix;
+	gboolean	display_status;
 };
 
 /* Class struct */
